from . import platforms as _  # noqa: F401
from .base import PlatformInstall
from .cli import SetupArgumentParser, prompts


def main():
    args = SetupArgumentParser().parse_args()

    selected_platform = args.platform or prompts.get_selection_from_user(
        "Platforms:", PlatformInstall.platforms()
    )
    if selected_platform is None:
        exit()

    setupper = PlatformInstall.from_key(selected_platform)

    if setupper is None:
        print(f"Platform {selected_platform!r} is not supported.")
        exit(1)

    if not args.silent:
        setupper.report()

        if not prompts.yn_prompt("Proceed?"):
            return

    success = setupper.run()
    if success:
        print("All done!")
    else:
        print("Something went wrong.")


if __name__ == "__main__":
    try:
        main()
    except RuntimeError as re:
        print(*re.args)
    except KeyboardInterrupt:
        pass



from __future__ import annotations

import os

from acconeer.exptool.setup.base import PlatformInstall, RequireFileContentStep, ShellCommandStep


@PlatformInstall.register
class Ubuntu_20_04(PlatformInstall):
    UDEV_RULE_FILE = "/etc/udev/rules.d/50-ft4222.rules"
    UDEV_RULE = (
        'SUBSYSTEM=="usb", ATTRS{idVendor}=="0403", ATTRS{idProduct}=="601c", MODE:="0666"\n'
    )

    def __init__(self) -> None:
        self.steps = [
            ShellCommandStep("sudo apt update".split()),
            ShellCommandStep("sudo apt install -y libxcb-xinerama0-dev".split()),
            ShellCommandStep(f"sudo usermod -a -G dialout {os.environ.get('USER')}".split()),
            RequireFileContentStep(self.UDEV_RULE_FILE, self.UDEV_RULE, sudo=True),
        ]

    @classmethod
    def get_key(cls) -> str:
        return "Ubuntu_20.04"

    def report(self) -> None:
        print(f"Setting up {self.get_key()!r} will do the following:")
        print()
        for step in self.steps:
            step.report()

    def run(self) -> bool:
        for step in self.steps:
            success = step.run()
            if not success:
                return False
        return True



from __future__ import annotations

import logging
from functools import partial
from typing import Any, Mapping, Optional

from PySide6.QtCore import Signal
from PySide6.QtWidgets import QVBoxLayout, QWidget

from acconeer.exptool import a121
from acconeer.exptool.a121._core import Criticality

from . import pidgets
from .sensor_config_editor import SensorConfigEditor
from .types import PidgetFactoryMapping
from .utils import VerticalGroupBox


log = logging.getLogger(__name__)


class SessionConfigEditor(QWidget):
    _session_config: Optional[a121.SessionConfig]
    _all_pidgets: list[pidgets.ParameterWidget]
    _server_info: Optional[a121.ServerInfo]

    sig_update = Signal(object)

    SPACING = 15

    SESSION_CONFIG_FACTORIES: PidgetFactoryMapping = {
        "sensor_id": pidgets.UpdateableComboboxParameterWidgetFactory(
            name_label_text="Sensor:",
            items=[],
        ),
        "update_rate": pidgets.OptionalFloatParameterWidgetFactory(
            name_label_text="Update rate:",
            limits=(0.1, 1e4),
            decimals=1,
            init_set_value=10.0,
            suffix="Hz",
            checkbox_label_text="Limit",
        ),
    }

    def __init__(self, parent: Optional[QWidget] = None) -> None:
        super().__init__(parent=parent)

        self._server_info = None

        self._session_config = None
        self._all_pidgets = []

        self.setLayout(QVBoxLayout(self))
        self.layout().setContentsMargins(0, 0, 0, 0)

        self._sensor_config_editor = SensorConfigEditor(self)

        self._sensor_config_editor.sig_update.connect(self._broadcast)

        # Session pidgets

        self.session_group_box = VerticalGroupBox("Session parameters", parent=self)
        self.session_group_box.layout().setSpacing(self.SPACING)
        self.layout().addWidget(self.session_group_box)

        self._session_config_pidgets: Mapping[str, pidgets.ParameterWidget] = {}
        for aspect, factory in self.SESSION_CONFIG_FACTORIES.items():
            pidget = factory.create(self.session_group_box)
            self.session_group_box.layout().addWidget(pidget)

            pidget.sig_parameter_changed.connect(
                partial(self._update_session_config_aspect, aspect)
            )

            self._all_pidgets.append(pidget)
            self._session_config_pidgets[aspect] = pidget

        self.layout().addWidget(self._sensor_config_editor)

    def set_data(self, session_config: Optional[a121.SessionConfig]) -> None:
        self._session_config = session_config
        if session_config is not None:
            self._sensor_config_editor.set_data(session_config.sensor_config)

    def sync(self) -> None:
        self._update_ui()
        self._sensor_config_editor.sync()

    def setEnabled(self, enabled: bool) -> None:
        super().setEnabled(enabled and self._session_config is not None)

    def _broadcast(self) -> None:
        self.sig_update.emit(self._session_config)

    def _handle_validation_results(self, results: list[a121.ValidationResult]) -> None:
        if results == []:
            for pidget in self._all_pidgets:
                pidget.set_note_text("")
        else:
            for result in results:
                self._handle_validation_result(result)

    def _update_session_config_aspect(self, aspect: str, value: Any) -> None:
        if self._session_config is None:
            raise TypeError("SessionConfig is None")
        try:
            setattr(self._session_config, aspect, value)
        except Exception as e:
            self._session_config_pidgets[aspect].set_note_text(e.args[0], Criticality.ERROR)
        else:
            self._handle_validation_results(self._session_config._collect_validation_results())

        self._broadcast()

    def _handle_validation_result(self, result: a121.ValidationResult) -> None:
        if result.aspect is None or self._session_config is None:
            return

        if result.source is self._session_config:
            pidget_map = self._session_config_pidgets
        else:
            return

        pidget_map[result.aspect].set_note_text(result.message, result.criticality)

    def _update_ui(self) -> None:
        if self._session_config is None:
            log.debug("could not update ui as SessionConfig is None")
            return

        self._session_config_pidgets["update_rate"].set_parameter(self._session_config.update_rate)
        self._session_config_pidgets["sensor_id"].set_parameter(self._session_config.sensor_id)

    def update_available_sensor_list(self, server_info: Optional[a121.ServerInfo]) -> None:
        if self._server_info == server_info:
            return

        self._server_info = server_info
        if server_info is None:
            self._session_config_pidgets["sensor_id"].update_items([])
        else:
            self._session_config_pidgets["sensor_id"].update_items(
                self._make_connected_sensor_list(server_info)
            )
        self._update_ui()

    def _make_connected_sensor_list(self, server_info: a121.ServerInfo) -> list[tuple[str, int]]:
        connected_sensors: list[tuple[str, int]] = []

        for k, v in server_info.sensor_infos.items():
            if v.connected:
                connected_sensors.append((str(k), k))

        return connected_sensors
